<?php

namespace ProductManagerUtil
{
    function exitOnFatalError($code, $reason = '')
    {
        $result = new \stdClass();
        //$result->result = null;

        if ($reason == '') {
            $result->error = $code;
        } else {
            $result->error = $code . ', ' . $reason;
        }

        echo json_encode($result);
        exit();
    }

    function writeHeaderAndCookie()
    {
        // change script name to index.php (Mage::getBaseUrl returns a wrong URL in BusinessKing_OutofStockSubscription extension)
        if (isset($_SERVER)) {
            $_SERVER['SCRIPT_FILENAME'] = str_replace($_SERVER['SCRIPT_NAME'], '/index.php', $_SERVER['SCRIPT_FILENAME']);
            $_SERVER['REQUEST_URI'] = '/index.php';
            $_SERVER['SCRIPT_NAME'] = '/index.php';
            $_SERVER['PHP_SELF'] = '/index.php';
        }

        //header
        header($_SERVER['SERVER_PROTOCOL'] . ' 202 Accepted');  //200 OK
        header('Cache-Control: no-store, private, no-cache, must-revalidate');
        header('Cache-Control: pre-check=0, post-check=0, max-age=0, max-stale=0', false);
        header('Expires: Sat, 26 Jul 1997 05:00:00 GMT');
        header('Expires: 0', false);
        header('Last-Modified: ' . gmdate('D, d M Y H:i:s') . ' GMT');
        header('Pragma: no-cache');
        header('Content-type: application/json');
        setcookie('EXTERNAL_NO_CACHE', '1');
        setcookie('nocache', '1');
        setcookie('no-cache', '1');
        setcookie('NO_CACHE', '1');
        setcookie('external_no_cache', '1');
    }

    function decipher($data, $key, $iv)
    {
        if (function_exists('openssl_decrypt')) {
            $plaintext = openssl_decrypt($data, 'aes-256-cbc', $key, OPENSSL_RAW_DATA, $iv);
        } else {
            // @codingStandardsIgnoreStart
            $td = @mcrypt_module_open(MCRYPT_RIJNDAEL_128, '', MCRYPT_MODE_CBC, '');
            @mcrypt_generic_init($td, $key, $iv);
            $plaintext = @mdecrypt_generic($td, $data);
            // @codingStandardsIgnoreEnd

            //remove PKCS7 padding
            $last = substr($plaintext, -1);
            $plaintext = substr($plaintext, 0, strlen($plaintext) - ord($last));

            // @codingStandardsIgnoreStart
            @mcrypt_generic_deinit($td);
            @mcrypt_module_close($td);
            // @codingStandardsIgnoreEnd
        }

        return $plaintext;
    }

    function getJSONFromPOSTFields($encryptionKey)
    {
        //check if all the fields are set
        if (!isset($_POST['c']) and !isset($_POST['iv'])) {
            exitOnFatalError('FATAL_ERROR_POST_FIELDS_MISSING');
        } else {
            //encrypted data
            if (function_exists('mcrypt_module_open')) {
                $iv = base64_decode($_POST['iv']);
                $cipherText = base64_decode($_POST['c']);
                $json = decipher($cipherText, base64_decode($encryptionKey), $iv);
                $jsonRpc = json_decode($json);

                if ($jsonRpc == null) {
                    exitOnFatalError('FATAL_ERROR_INVALID_JSON', $json);
                }
            } else {
                $json = $_POST['c'];
                $jsonRpc = json_decode($json);

                if ($jsonRpc == null) {
                    exitOnFatalError('FATAL_ERROR_INVALID_JSON_ENCRYPTION_NOT_SUPPORTED');
                }
            }

            return $jsonRpc;
        }
    }

    function printCompressedDataHeader($dataLength)
    {
        echo 'P';
        echo 'M';
        echo '1';
        echo '0';

        $a = ($dataLength << 24) & 0xFF;
        $b = ($dataLength << 16) & 0xFF;
        $c = ($dataLength << 8) & 0xFF;
        $d = $dataLength & 0xFF;

        echo chr($a);
        echo chr($b);
        echo chr($c);
        echo chr($d);
    }

    function writeJSON($jsonRpcResult)
    {
        if (function_exists('gzcompress')) {
            $data = json_encode($jsonRpcResult);
            printCompressedDataHeader(strlen($data));
            echo gzcompress($data, 6);
        } else {
            echo json_encode($jsonRpcResult);
        }
    }
}